import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

// a function that draws a rectrangle for each row in the table:

const render = (data) => {
  // value accessors:
  const xValue = (d) => d.population;
  const yValue = (d) => d.country;
  // adding margins:
  const margin = {
    top: 30,
    right: 40,
    bottom: 20,
    left: 100,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  // setting up linear and band scales:
  const xScale = scaleLinear() // we can envoke scale linear as a function to create an instance of a d3 scale
    .domain([0, max(data, xValue)])
    .range([0, innerWidth]); // this means the maximum population should map to the maximum bar width
  // set a y scale too
  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, innerHeight]) // allows the data elements to be arranged from top to bottom
    .padding(0.1);

  // lets append our rectangles to g instead of svg so that we have a margin...
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  // setting up the axes:
  g.append('g').call(axisLeft(yScale));
  g.append('g')
    .call(axisBottom(xScale))
    .attr('transform', `translate(0,${innerHeight})`);

  // change rect to circles
  g.selectAll('circle')
    .data(data) // d3 selects all existing rectangles ( this is the d3 data join)
    .enter()
    .append('circle')
    .attr('cy', (d) => yScale(yValue(d)))
    .attr('cx', (d) => xScale(xValue(d)))
    .attr('r', yScale.bandwidth());


};

// loading and parsing data:
csv('data.csv').then((data) => {
  data.forEach((d) => {
    d.population = +d.population * 1000;
  });
  // console.log(data);
  // lets define a function to draw the rectangles for each row
  render(data);
});
//  csv() is a promis that resolves when this data is loaded
//  this csv() function of D3 makes an http request ,
//  loads the csv string from data.csv
//  then parses the csv into an array of objects

//keys of each object represent the columns
// values represent the values in the table
